/* GthGridView  */

iconview.icon {
	background: @theme_bg_color;
	color: @theme_fg_color;
	border-radius: 4px;
}

iconview.icon:selected,
iconview.icon:selected:focus {
	background-color: @theme_selected_bg_color;
	color: @theme_selected_fg_color;
}

iconview.icon-effect {
	background-color: alpha(@theme_bg_color, 0.5);
	color: @theme_fg_color;
	border-radius: 0;
}

iconview.icon-effect:selected,
iconview.icon-effect:selected:focus {
	background-color: alpha(@theme_selected_bg_color, 0.5);
	color: @theme_selected_fg_color;
	border-radius: 0;
}
